@@ -10,8 +10,14 @@ as.IDate.default <- function(x, ..., tz = attr(x, "tzone")) {
   as.IDate(as.Date(x, tz = tz, ...))
 }
 
-as.IDate.numeric <- function(x, ...) {
-  structure(as.integer(x), class=c("IDate","Date"))
+as.IDate.numeric <- function(x, origin = "1970-01-01", ...) {
+  if (origin=="1970-01-01") {
+    # standard epoch
+    structure(as.integer(x), class=c("IDate","Date"))
+  } else {
+    # only call expensive as.IDate.character if we have to
+    as.IDate(origin, ...) + as.integer(x)
+  }
 }
 
 as.IDate.Date <- function(x, ...) {
@@ -65,8 +71,7 @@ round.IDate <- function (x, digits=c("weeks", "months", "quarters", "years"), ..
     return(e1)
   if (inherits(e1, "difftime") || inherits(e2, "difftime"))
     stop("difftime objects may not be added to IDate. Use plain integer instead of difftime.")
-  if ( (storage.mode(e1)=="double" && isReallyReal(e1)) ||
-       (storage.mode(e2)=="double" && isReallyReal(e2)) ) {
+  if (isReallyReal(e1) || isReallyReal(e2)) {
     return(`+.Date`(e1,e2))
     # IDate doesn't support fractional days; revert to base Date
   }
@@ -84,7 +89,7 @@ round.IDate <- function (x, digits=c("weeks", "months", "quarters", "years"), ..
     stop("unary - is not defined for \"IDate\" objects")
   if (inherits(e2, "difftime"))
     stop("difftime objects may not be subtracted from IDate. Use plain integer instead of difftime.")
-  if ( storage.mode(e2)=="double" && isReallyReal(e2) ) {
+  if ( isReallyReal(e2) ) {
     return(`-.Date`(as.Date(e1),as.Date(e2)))
     # IDate doesn't support fractional days so revert to base Date
   }
@@ -103,21 +108,27 @@ round.IDate <- function (x, digits=c("weeks", "months", "quarters", "years"), ..
 as.ITime <- function(x, ...) UseMethod("as.ITime")
 
 as.ITime.default <- function(x, ...) {
-  as.ITime(as.POSIXlt(x, ...))
+  as.ITime(as.POSIXlt(x, ...), ...)
 }
 
 as.ITime.POSIXct <- function(x, tz = attr(x, "tzone"), ...) {
   if (is.null(tz)) tz = "UTC"
-  if (tz %in% c("UTC", "GMT")) as.ITime(unclass(x), ...) else as.ITime(as.POSIXlt(x, tz = tz, ...))
+  if (tz %in% c("UTC", "GMT")) as.ITime(unclass(x), ...) else as.ITime(as.POSIXlt(x, tz = tz, ...), ...)
 }
 
-as.ITime.numeric <- function(x, ...) {
-  structure(as.integer(x) %% 86400L, class = "ITime")
+as.ITime.numeric <- function(x, ms = 'truncate', ...) {
+  secs = switch(ms,
+                'truncate' = as.integer(x),
+                'nearest' = as.integer(round(x)),
+                'ceil' = as.integer(ceiling(x)),
+                stop("Valid options for ms are 'truncate', ",
+                     "'nearest', and 'ceil'."))
+  structure(secs %% 86400L, class = "ITime")
 }
 
 as.ITime.character <- function(x, format, ...) {
   x <- unclass(x)
-  if (!missing(format)) return(as.ITime(strptime(x, format = format, ...)))
+  if (!missing(format)) return(as.ITime(strptime(x, format = format, ...), ...))
   # else allow for mixed formats, such as test 1189 where seconds are caught despite varying format
   y <- strptime(x, format = "%H:%M:%OS", ...)
   w <- which(is.na(y))
@@ -133,18 +144,35 @@ as.ITime.character <- function(x, format, ...) {
     new <- strptime(x[w], format = f, ...)
     nna <- !is.na(new)
     if (any(nna)) {
-      y[ w[nna] ] <- new
+      y[ w[nna] ] <- new[nna]
       w <- w[!nna]
     }
   }
-  return(as.ITime(y))
+  return(as.ITime(y, ...))
 }
 
-as.ITime.POSIXlt <- function(x, ...) {
-  structure(with(x, as.integer(sec) + min * 60L + hour * 3600L),
+as.ITime.POSIXlt <- function(x, ms = 'truncate', ...) {
+  secs = switch(ms,
+                'truncate' = as.integer(x$sec),
+                'nearest' = as.integer(round(x$sec)),
+                'ceil' = as.integer(ceiling(x$sec)),
+                stop("Valid options for ms are 'truncate', ",
+                     "'nearest', and 'ceil'."))
+  structure(with(x, secs + min * 60L + hour * 3600L),
         class = "ITime")
 }
 
+as.ITime.times <- function(x, ms = 'truncate', ...) {
+  secs <- 86400 * (unclass(x) %% 1)
+  secs = switch(ms,
+                'truncate' = as.integer(secs),
+                'nearest' = as.integer(round(secs)),
+                'ceil' = as.integer(ceiling(secs)),
+                stop("Valid options for ms are 'truncate', ",
+                     "'nearest', and 'ceil'."))
+  structure(secs, class = "ITime")
+}
+
 as.character.ITime <- format.ITime <- function(x, ...) {
   # adapted from chron's format.times
   # Fix for #811. Thanks to @StefanFritsch for the code snippet
@@ -234,14 +262,6 @@ as.POSIXlt.ITime <- function(x, ...) {
   as.POSIXlt(as.POSIXct(x, ...))
 }
 
-as.ITime.times <- function(x, ...) {
-  x <- unclass(x)
-  daypart <- x - floor(x)
-  secs <- as.integer(round(daypart * 86400))
-  structure(secs,
-        class = "ITime")
-}
-
 ###################################################################
 # Date - time extraction functions
 #   Adapted from Hadley Wickham's routines cited below to ensure
@@ -252,9 +272,30 @@ as.ITime.times <- function(x, ...) {
 #   lubridate routines do not return integer values.
 ###################################################################
 
-second  <- function(x) as.integer(as.POSIXlt(x)$sec)
-minute  <- function(x) as.POSIXlt(x)$min
-hour    <- function(x) as.POSIXlt(x)$hour
+second  <- function(x) {
+  if (inherits(x,'POSIXct') && identical(attr(x,'tzone'),'UTC')) {
+    # if we know the object is in UTC, can calculate the hour much faster
+    as.integer(x) %% 60L
+  } else {
+    as.integer(as.POSIXlt(x)$sec)
+  }
+}
+minute  <- function(x) {
+  if (inherits(x,'POSIXct') && identical(attr(x,'tzone'),'UTC')) {
+    # ever-so-slightly faster than x %% 3600L %/% 60L
+    as.integer(x) %/% 60L %% 60L
+  } else {
+    as.POSIXlt(x)$min
+  }
+}
+hour <- function(x) {
+  if (inherits(x,'POSIXct') && identical(attr(x,'tzone'),'UTC')) {
+    # ever-so-slightly faster than x %% 86400L %/% 3600L
+    as.integer(x) %/% 3600L %% 24L
+  } else {
+    as.POSIXlt(x)$hour
+  }
+}
 yday    <- function(x) as.POSIXlt(x)$yday + 1L
 wday    <- function(x) (unclass(as.IDate(x)) + 4L) %% 7L + 1L
 mday    <- function(x) as.POSIXlt(x)$mday
