@@ -157,9 +157,11 @@ as.data.table.list <- function(x, keep.rownames=FALSE, ...) {
 # tests and as.data.table.data.frame) I've commented #527
 # for now. This addresses #1078 and #1128
 .resetclass <- function(x, class) {
+  if (length(class)!=1L)
+    stop("class must be length 1") # nocov
   cx = class(x)
-  n  = chmatch(class, cx)
-  cx = unique( c("data.table", "data.frame", tail(cx, length(cx)-n)) )
+  n  = chmatch(class, cx)   # chmatch accepts legth(class)>1 but next line requires length(n)==1
+  unique( c("data.table", "data.frame", tail(cx, length(cx)-n)) )
 }
 
 as.data.table.data.frame <- function(x, keep.rownames=FALSE, ...) {
