@@ -2,7 +2,7 @@ setkey <- function(x, ..., verbose=getOption("datatable.verbose"), physical=TRUE
 {
   if (is.character(x)) stop("x may no longer be the character name of the data.table. The possibility was undocumented and has been removed.")
   cols = as.character(substitute(list(...))[-1L])
-  if (!length(cols)) cols=colnames(x)
+  if (!length(cols)) { cols=colnames(x) }
   else if (identical(cols,"NULL")) cols=NULL
   setkeyv(x, cols, verbose=verbose, physical=physical)
 }
@@ -51,18 +51,27 @@ setkeyv <- function(x, cols, verbose=getOption("datatable.verbose"), physical=TR
     # remove backticks from cols
     cols <- gsub("`", "", cols)
     miss = !(cols %in% colnames(x))
-    if (any(miss)) stop("some columns are not in the data.table: " %+% cols[miss])
+    if (any(miss)) stop("some columns are not in the data.table: ", paste(cols[miss], collapse=","))
   }
 
   ## determine, whether key is already present:
   if (identical(key(x),cols)) {
-    ## key is present, nothing needs to be done
+    if (!physical) {
+      ## create index as integer() because already sorted by those columns
+      if (is.null(attr(x,"index",exact=TRUE))) setattr(x, "index", integer())
+      setattr(attr(x,"index",exact=TRUE), paste0("__", cols, collapse=""), integer())
+    }
     return(invisible(x))
   } else if(identical(head(key(x), length(cols)), cols)){
-    ## key is present but x has a longer key. No sorting needed, only attribute is changed to shorter key.
-    setattr(x,"sorted",cols)
+    if (!physical) {
+      ## create index as integer() because already sorted by those columns
+      if (is.null(attr(x,"index",exact=TRUE))) setattr(x, "index", integer())
+      setattr(attr(x,"index",exact=TRUE), paste0("__", cols, collapse=""), integer())
+    } else {
+      ## key is present but x has a longer key. No sorting needed, only attribute is changed to shorter key.
+      setattr(x,"sorted",cols)
+    }
     return(invisible(x))
-    ## maybe additional speedup can be achieved if part of the key is already present?
   }
 
   if (".xi" %chin% names(x)) stop("x contains a column called '.xi'. Conflicts with internal use by data.table.")
@@ -72,7 +81,8 @@ setkeyv <- function(x, cols, verbose=getOption("datatable.verbose"), physical=TR
   }
   if (!is.character(cols) || length(cols)<1L) stop("'cols' should be character at this point in setkey")
   if (verbose) {
-    tt = system.time(o <- forderv(x, cols, sort=TRUE, retGrp=FALSE))  # system.time does a gc, so we don't want this always on, until refcnt is on by default in R
+    tt = suppressMessages(system.time(o <- forderv(x, cols, sort=TRUE, retGrp=FALSE)))  # system.time does a gc, so we don't want this always on, until refcnt is on by default in R
+    # suppress needed for tests 644 and 645 in verbose mode
     cat("forder took", tt["user.self"]+tt["sys.self"], "sec\n")
   } else {
     o <- forderv(x, cols, sort=TRUE, retGrp=FALSE)
@@ -85,7 +95,7 @@ setkeyv <- function(x, cols, verbose=getOption("datatable.verbose"), physical=TR
   setattr(x,"index",NULL)   # TO DO: reorder existing indexes likely faster than rebuilding again. Allow optionally. Simpler for now to clear.
   if (length(o)) {
     if (verbose) {
-      tt = system.time(.Call(Creorder,x,o))
+      tt = suppressMessages(system.time(.Call(Creorder,x,o)))
       cat("reorder took", tt["user.self"]+tt["sys.self"], "sec\n")
     } else {
       .Call(Creorder,x,o)
@@ -102,12 +112,21 @@ key <- function(x) attr(x,"sorted",exact=TRUE)
 indices <- function(x, vectors = FALSE) {
   ans = names(attributes(attr(x,"index",exact=TRUE)))
   if (is.null(ans)) return(ans) # otherwise character() gets returned by next line
-  ans <- gsub("^__","",ans)
+  ans <- gsub("^__","",ans)     # the leading __ is internal only, so remove that in result
   if (isTRUE(vectors))
     ans <- strsplit(ans, "__", fixed = TRUE)
   ans
 }
 
+getindex <- function(x, name) {
+  # name can be "col", or "col1__col2", or c("col1","col2")
+  ans = attr(attr(x, 'index'), paste0("__",name,collapse=""), exact=TRUE)
+  if (!is.null(ans) && (!is.integer(ans) || (length(ans)!=nrow(x) && length(ans)!=0L))) {
+    stop("Internal error: index '",name,"' exists but is invalid")   # nocov
+  }
+  ans
+}
+
 "key<-" <- function(x,value) {
   warning("The key(x)<-value form of setkey can copy the whole table. This is due to <- in R itself. Please change to setkeyv(x,value) or setkey(x,...) which do not copy and are faster. See help('setkey'). You can safely ignore this warning if it is inconvenient to change right now. Setting options(warn=2) turns this warning into an error, so you can then use traceback() to find and change your key<- calls.")
   setkeyv(x,value)
@@ -182,8 +201,8 @@ forderv <- function(x, by=seq_along(x), retGrp=FALSE, sort=TRUE, order=1L, na.la
       if (anyNA(w)) stop("'by' contains '",by[is.na(w)][1],"' which is not a column name")
       by = w
     }
-    else if (typeof(by)=="double" && isReallyReal(by)) {
-      stop("'by' is type 'double' but one or more items in it are not whole integers")
+    else if (isReallyReal(by)) {
+      stop("'by' is type 'double' and one or more items in it are not whole integers")
     }
     by = as.integer(by)
     if ( (length(order) != 1L && length(order) != length(by)) || any(!order %in% c(1L, -1L)) )
@@ -240,13 +259,10 @@ forder <- function(x, ..., na.last=TRUE, decreasing=FALSE)
   o
 }
 
-fsort <- function(x, decreasing = FALSE, na.last = FALSE, internal=FALSE, verbose=FALSE, ...)
+fsort <- function(x, decreasing=FALSE, na.last=FALSE, internal=FALSE, verbose=FALSE, ...)
 {
   containsNAs <- FALSE
-  if (typeof(x)=="double" && !decreasing) {
-    containsNAs <- anyNA(x) ## just do this if all other conditions are met since it is relatively expensive
-  }
-  if(typeof(x)=="double" && !decreasing && !containsNAs){
+  if (typeof(x)=="double" && !decreasing && !(containsNAs<-anyNA(x))) {
       if (internal) stop("Internal code should not be being called on type double")
       return(.Call(Cfsort, x, verbose))
   }
@@ -255,13 +271,13 @@ fsort <- function(x, decreasing = FALSE, na.last = FALSE, internal=FALSE, verbos
     # The only places internally we use fsort internally (3 calls, all on integer) have had internal=TRUE added for now.
     # TODO: implement integer and character in Cfsort and remove this branch and warning
     if (!internal){
-      if(typeof(x) != "double") warning("Input is not a vector of type double. New parallel sort has only been done for double vectors so far. Using one thread.")
-      if(decreasing)  warning("New parallel sort has not been implemented for decreasing=TRUE so far. Using one thread.")
-      if(containsNAs) warning("New parallel sort has not been implemented for vectors containing NA values so far. Using one thread.")
+      if (typeof(x)!="double") warning("Input is not a vector of type double. New parallel sort has only been done for double vectors so far. Using one thread.")
+      if (decreasing)  warning("New parallel sort has not been implemented for decreasing=TRUE so far. Using one thread.")
+      if (containsNAs) warning("New parallel sort has not been implemented for vectors containing NA values so far. Using one thread.")
     }
-    orderArg = if(decreasing) -1 else 1
+    orderArg = if (decreasing) -1 else 1
     o = forderv(x, order=orderArg, na.last=na.last)
-    return( if (length(o)) x[o] else x )   # TO DO: document this shortcut for already-sorted
+    return( if (length(o)) x[o] else x )
   }
 }
 
@@ -310,7 +326,7 @@ setorderv <- function(x, cols, order=1L, na.last=FALSE)
     # remove backticks from cols
     cols <- gsub("`", "", cols)
     miss = !(cols %in% colnames(x))
-    if (any(miss)) stop("some columns are not in the data.table: " %+% cols[miss])
+    if (any(miss)) stop("some columns are not in the data.table: ", paste(cols[miss], collapse=","))
   }
   if (".xi" %in% colnames(x)) stop("x contains a column called '.xi'. Conflicts with internal use by data.table.")
   for (i in cols) {
@@ -390,13 +406,14 @@ CJ <- function(..., sorted = TRUE, unique = FALSE)
   }
   setattr(l, "row.names", .set_row_names(length(l[[1L]])))
   setattr(l, "class", c("data.table", "data.frame"))
-
-  if (is.null(vnames <- names(l)))
-    vnames = vector("character", length(l))
-  if (any(tt <- vnames == "")) {
-    vnames[tt] = paste0("V", which(tt))
-    setattr(l, "names", vnames)
+  if (getOption("datatable.CJ.names", FALSE)) {  # option added v1.11.6. Change to TRUE in v1.12.0
+    vnames = name_dots(...)$vnames
+  } else {
+    if (is.null(vnames <- names(l))) vnames = paste0("V", seq_len(length(l)))
+    else if (any(tt <- vnames=="")) vnames[tt] = paste0("V", which(tt))
   }
+  setattr(l, "names", vnames)
+
   l <- alloc.col(l)  # a tiny bit wasteful to over-allocate a fixed join table (column slots only), doing it anyway for consistency, and it's possible a user may wish to use SJ directly outside a join and would expect consistent over-allocation.
   if (sorted) {
     if (!dups) setattr(l, 'sorted', names(l))
