@@ -455,7 +455,7 @@ SEXP setNumericRounding(SEXP droundArg)
 // init.c has initial call with default of 2
 {
   if (!isInteger(droundArg) || LENGTH(droundArg)!=1) error("Must an integer or numeric vector length 1");
-  if (INTEGER(droundArg)[0] < 0 || INTEGER(droundArg)[0] > 2) error("Must be 2 (default) or 1 or 0");
+  if (INTEGER(droundArg)[0] < 0 || INTEGER(droundArg)[0] > 2) error("Must be 2, 1 or 0");
   dround = INTEGER(droundArg)[0];
   dmask1 = dround ? 1 << (8*dround-1) : 0;
   dmask2 = 0xffffffffffffffff << dround*8;
@@ -467,6 +467,12 @@ SEXP getNumericRounding()
   return ScalarInteger(dround);
 }
 
+int getNumericRounding_C()
+// for use in uniqlist.c
+{
+  return dround;
+}
+
 static union {
   double d;
   unsigned long long ull;
@@ -1445,18 +1451,20 @@ SEXP isOrderedSubset(SEXP x, SEXP nrow)
 */
 
 SEXP isReallyReal(SEXP x) {
-  int n, i=0;
-  SEXP ans;
-  if (!isReal(x))
-    error("x must be of type double.");
-  n = length(x);
-  ans = PROTECT(allocVector(LGLSXP, 1));
-  while (i<n &&
-      ( ISNA(REAL(x)[i]) ||
-      ( R_FINITE(REAL(x)[i]) && REAL(x)[i] == (int)(REAL(x)[i])))) {
-    i++;
+  SEXP ans = PROTECT(allocVector(INTSXP, 1));
+  INTEGER(ans)[0] = 0;
+  // return 0 (FALSE) when not type double, or is type double but contains integers
+  // used to error if not passed type double but this needed extra is.double() calls in calling R code
+  // which needed a repeat of the argument. Hence simpler and more robust to return 0 when not type double.
+  if (isReal(x)) {
+    int n=length(x), i=0;
+    while (i<n &&
+        ( ISNA(REAL(x)[i]) ||
+        ( R_FINITE(REAL(x)[i]) && REAL(x)[i] == (int)(REAL(x)[i])))) {
+      i++;
+    }
+    if (i<n) INTEGER(ans)[0] = i+1;  // return the location of first element which is really real; i.e. not an integer
   }
-  LOGICAL(ans)[0] = (i<n);
   UNPROTECT(1);
   return(ans);
 }
